/**
   chat: a TCP/IP chatting program
   Copyright (C) 2006  Idan Nof

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Idan Nof can be reached via -

   email: idannof (at) hotmail (dot) com
   web  : http://www.geocities.com/idannof
   paper: 3 Brande st.,
		  Petach-Tikva, 49600,
		  Israel
*/

package Client;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import Util.Util;

/**
 * A client initiating a single connection and sends/receives messages
 */
public class Client {
	/**
	 * An output for this socket
	 */
	private BufferedWriter sockOut = null;

	/**
	 * An input for this socket
	 */
	private BufferedReader sockIn = null;

	/**
	 * The Socket on which the server communicating.
	 */
	private Socket socket = null;

	/**
	 * A thread that listens on the network for server messages 
	 */
	private ServerEchoListener serverEchoListener = null;

	/**
	 * a thread that sits on the shared fifo pipe and sends messages to the server
	 */
	private UserSend userSend = null;
	
	/**
	 * A predicate that says whether we are connected
	 */
	private boolean isConnected = false;

	/**
	 * Initiate a connection to a server
	 * @param server_host The server hostname
	 * @param server_port The server port number
	 * @return
	 */
	public final Socket connect(String server_host, int server_port) {
		Socket socket = null;
		try {
			socket = new Socket(server_host, server_port);
		} catch (UnknownHostException e) {
			System.out.println("UnknownHostException at connect()");
			return null;
		} catch (IOException e) {
			System.out.println(
				"IOException at connect(), Is there a server listening?");
			return null;
		}

		System.out.println("Connected to server.");
		Util.printFormattedSocketName(socket);
		return socket;
	}

	/**
	 * A procedure that sets the global 'read' variable to wrap the socket InputStreamReader,
	 * and 'write' to wrap the socket OutputStreamWriter.
	 */
	private final void doGetReaderWriter() {
		try {
			InputStream inputStream = socket.getInputStream();
			InputStreamReader inputStreamReader =
				new InputStreamReader(inputStream);
			sockIn = new BufferedReader(inputStreamReader);

			OutputStream outputStream = socket.getOutputStream();
			OutputStreamWriter outputStreamWriter =
				new OutputStreamWriter(outputStream);
			sockOut = new BufferedWriter(outputStreamWriter);

		} catch (IOException e) {
			System.out.println("Cannot get input and output streams.");
		}
	}

	/**
	 * A frontend to connect()
	 * Initiate a connection to a server and start the 
	 *
	 * @param server_host A server hostname to connect to
	 * @param server_port A server port number to connect to
	 * @return A Socket for the connection
	 */
	public final Socket doConnect(String server_host, int server_port) {
		if (isConnected) {
			System.out.println("Already connected.");
			return null;
		}
		
		socket = connect(server_host, server_port);
		if (socket == null) {
			System.out.println(
				"Unsuccessful connection - invalid socket returned.");
		}

		doGetReaderWriter();
		serverEchoListener = new ServerEchoListener(sockIn);
		serverEchoListener.start();

		userSend = new UserSend(sockOut);
		userSend.start();
		
		setConnected(true);
		return socket;
	}

	/**
	 * Cleanup; close socket and quit running threads.
	 */
	public final void doDisconnect() {
		if (!isConnected()) {
			System.out.println("not connected.");
			return;
		}
		
		userSend.interrupt();
		userSend = null;

		serverEchoListener.interrupt();
		serverEchoListener = null;

		try {
			socket.close();
			socket = null;
		} catch (IOException e) {
			System.out.println("Cannot close socket.");
		}
		
		setConnected(false);
	}

	/**
	 * Initiate a new client instance
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		Client client = new Client();
		new Gui(client).show();
	}
	
	/**
	 * @return
	 */
	public boolean isConnected() {
		return isConnected;
	}

	/**
	 * @param b
	 */
	public void setConnected(boolean b) {
		isConnected = b;
	}

}
