/**
   chat: a TCP/IP chatting program
   Copyright (C) 2006  Idan Nof

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Idan Nof can be reached via -

   email: idannof (at) hotmail (dot) com
   web  : http://www.geocities.com/idannof
   paper: 3 Brande st.,
		  Petach-Tikva, 49600,
		  Israel
*/
package Server;
import java.net.InetAddress;

/**
 * Each instance of the User class is an end-user, working as a client.
 * 
 * note: user is uniquily identified by both its internet address and its name.
 */
public class User {
	/**
	 * The internet address of the user.
	 * 
	 * Note: for security reason, this field is automatically filled with the address of
	 * the sending client. That is, a user cannot route the server packets delivered to 
	 * him to a different computer by declaring a different IP address. 
	 */
	private final InetAddress inetAddress;

	/**
	 * The name of the user.
	 * Initialized in the constructor.
	 */
	private final String userName;
	
	/**
	 * The socket this user is attached to.
	 */
	private SocketThread socketThread;

	/**
	 * A constructor. All of the identifying user fields are set here,
	 * such as IP address, etc.
	 * @param inetAddress The address of the user.
	 * @param userName The name of the user.
	 */
	User(InetAddress inetAddress, String userName) {
		this.inetAddress = inetAddress;
		this.userName = userName;
	}

	/**
	 * Returns the name of the user
	 * @return the name of the user
	 */
	public String toString() {
		return userName;
	}

	/**
	 * Returns the internet address of the user
	 * @return the internet address of the user
	 */
	public InetAddress getInetAddress() {
		return inetAddress;
	}

	/**
	 * An overriden equals() for this object.
	 * 
	 * Two user instances with the same internet address and the same name,
	 * will be regarded as being equal.
	 */
	public boolean equals(Object obj) {
		User u = (User) obj;
		boolean isEqual =
			getInetAddress().equals(u.getInetAddress())
				&& this.userName.equals(u.toString());

		return isEqual;
	}
	
	/**
	 * An overriden hashCode() for this object.
	 * 
	 * Two user instances with the same internet address and the same name, 
	 * will receive the same hash code.
	 */
	public int hashCode() {
		return inetAddress.hashCode() ^ userName.hashCode();
	}

	/**
	 * Returns the SocketThread associated with this user
	 * @return the SocketThread associated with this user
	 */
	public SocketThread getSocketThread() {
		return socketThread;
	}

	/**
	 * Set the SocketThread associated with this user
	 * @param thread The SocketThread to save
	 */
	public void setSocketThread(SocketThread thread) {
		socketThread = thread;
	}

}
